@@ -1,28 +1,45 @@
-#define __GNOME_PRINT_TRANSPORT_C__
-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Abstract base class for transport providers
+ *  gnome-print-transport.c: Abstract base class for transport providers
  *
- * Authors:
- *   Raph Levien (raph@acm.org)
- *   Miguel de Icaza (miguel@kernel.org)
- *   Lauris Kaplinski <lauris@ximian.com>
- *   Chema Celorio (chema@celorio.com)
+ *  This program is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public License
+ *  as published by the Free Software Foundation; either version 2 of
+ *  the License, or (at your option) any later version.
  *
- * Copyright (C) 1999-2001 Ximian, Inc. and authors
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Library General Public License for more details.
  *
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ *  Authors:
+ *    Raph Levien <raph@acm.org>
+ *    Miguel de Icaza <miguel@kernel.org>
+ *    Lauris Kaplinski <lauris@helixcode.com>
+ *    Chema Celorio <chema@celorio.com>
+ *
+ *  Copyright 2000-2003 Ximian, Inc. and authors
  */
 
+#define GNOME_PRINT_UNSTABLE_API
+
+#include <config.h>
 #include <string.h>
 #include <locale.h>
+
 #include <gmodule.h>
-#include "gnome-print.h"
-#include "gnome-print-transport.h"
+#include <libgnomeprint/gnome-print.h>
+#include <libgnomeprint/gnome-print-transport.h>
+#include <stdio.h>
 
 static void gnome_print_transport_class_init (GnomePrintTransportClass *klass);
 static void gnome_print_transport_init (GnomePrintTransport *transport);
-
 static void gnome_print_transport_finalize (GObject *object);
+static gint gnome_print_transport_real_print_file (GnomePrintTransport *transport, const guchar *file_name);
 
 static GnomePrintTransport *gnome_print_transport_create (gpointer get_type, GnomePrintConfig *config);
 
@@ -53,11 +70,13 @@ gnome_print_transport_class_init (GnomePrintTransportClass *klass)
 {
 	GObjectClass *object_class;
 
-	object_class = (GObjectClass*) klass;
+	object_class           = (GObjectClass*) klass;
 
-	parent_class = g_type_class_peek_parent (klass);
+	parent_class           = g_type_class_peek_parent (klass);
 
 	object_class->finalize = gnome_print_transport_finalize;
+
+	klass->print_file      = gnome_print_transport_real_print_file;
 }
 
 static void
@@ -88,18 +107,19 @@ gnome_print_transport_finalize (GObject *object)
 gint
 gnome_print_transport_construct (GnomePrintTransport *transport, GnomePrintConfig *config)
 {
+	GnomePrintReturnCode retval = GNOME_PRINT_ERROR_UNKNOWN;
+	
 	g_return_val_if_fail (transport != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (GNOME_IS_PRINT_TRANSPORT (transport), GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (config != NULL, GNOME_PRINT_ERROR_UNKNOWN);
-
 	g_return_val_if_fail (transport->config == NULL, GNOME_PRINT_ERROR_UNKNOWN);
 
 	transport->config = gnome_print_config_ref (config);
 
 	if (GNOME_PRINT_TRANSPORT_GET_CLASS (transport)->construct)
-		GNOME_PRINT_TRANSPORT_GET_CLASS (transport)->construct (transport);
+		retval = GNOME_PRINT_TRANSPORT_GET_CLASS (transport)->construct (transport);
 
-	return TRUE;
+	return retval;
 }
 
 gint
@@ -115,7 +135,7 @@ gnome_print_transport_open (GnomePrintTransport *transport)
 	ret = GNOME_PRINT_OK;
 
 	if (GNOME_PRINT_TRANSPORT_GET_CLASS (transport)->open)
-		GNOME_PRINT_TRANSPORT_GET_CLASS (transport)->open (transport);
+		ret = GNOME_PRINT_TRANSPORT_GET_CLASS (transport)->open (transport);
 
 	if (ret == GNOME_PRINT_OK) {
 		transport->opened = TRUE;
@@ -136,7 +156,7 @@ gnome_print_transport_close (GnomePrintTransport *transport)
 	ret = GNOME_PRINT_OK;
 
 	if (GNOME_PRINT_TRANSPORT_GET_CLASS (transport)->close)
-		GNOME_PRINT_TRANSPORT_GET_CLASS (transport)->close (transport);
+		ret = GNOME_PRINT_TRANSPORT_GET_CLASS (transport)->close (transport);
 
 	if (ret == GNOME_PRINT_OK) {
 		transport->opened = FALSE;
@@ -164,7 +184,7 @@ gint
 gnome_print_transport_printf (GnomePrintTransport *transport, const char *format, ...)
 {
 	va_list arguments;
-	const char *loc;
+	char *loc;
 	gchar *buf;
 	gint ret;
 
@@ -173,8 +193,9 @@ gnome_print_transport_printf (GnomePrintTransport *transport, const char *format
 	g_return_val_if_fail (format != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 	g_return_val_if_fail (transport->opened, GNOME_PRINT_ERROR_UNKNOWN);
 
-	loc = setlocale (LC_NUMERIC, NULL);
+	loc = g_strdup (setlocale (LC_NUMERIC, NULL));
 	setlocale (LC_NUMERIC, "C");
+#warning: We should not be using setlocale here.
 
 	va_start (arguments, format);
 	buf = g_strdup_vprintf (format, arguments);
@@ -187,57 +208,115 @@ gnome_print_transport_printf (GnomePrintTransport *transport, const char *format
 	g_free (buf);
 
 	setlocale (LC_NUMERIC, loc);
+	g_free (loc);
 
 	return ret;
 }
 
+
+gint
+gnome_print_transport_print_file (GnomePrintTransport *transport, const guchar *file_name)
+{
+	int ret;
+	
+	g_return_val_if_fail (transport != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (file_name != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (GNOME_IS_PRINT_TRANSPORT (transport), GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (transport->config != NULL, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (!transport->opened, GNOME_PRINT_ERROR_UNKNOWN);
+
+	ret = GNOME_PRINT_OK;
+
+	if (GNOME_PRINT_TRANSPORT_GET_CLASS (transport)->print_file)
+		ret = GNOME_PRINT_TRANSPORT_GET_CLASS (transport)->print_file (transport, file_name);
+
+	return ret;
+}
+
+
+static GnomePrintTransport *
+gnome_print_transport_new_from_module_name (const gchar *module_name, GnomePrintConfig *config)
+{
+	static GHashTable *modules;
+	GnomePrintTransport *transport;
+	gpointer get_type;
+	GModule *module;
+	gchar *path = NULL;
+	gboolean insert = FALSE;
+
+	if (!modules)
+		modules = g_hash_table_new (g_str_hash, g_str_equal);
+	module = g_hash_table_lookup (modules, module_name);
+
+	if (!module) {
+		insert = TRUE;
+		path = g_module_build_path (GNOME_PRINT_MODULES_DIR "/transports", module_name);
+		module = g_module_open (path, G_MODULE_BIND_LAZY);
+	}
+
+	if (!module) {
+		insert = TRUE;
+		g_free (path);
+		path = g_module_build_path (GNOME_PRINT_MODULES_DIR, module_name);
+		module = g_module_open (path, G_MODULE_BIND_LAZY);
+	}
+	
+	if (!module) {
+		g_warning ("Could not open %s\n", path);
+		g_free (path);
+		return NULL;
+	}
+
+	if (insert)
+		g_hash_table_insert (modules, g_strdup (module_name), module);
+
+	if (!g_module_symbol (module, "gnome_print__transport_get_type", &get_type)) {
+		g_warning ("Missing gnome_print__transport_get_type in %s\n", path);
+		g_module_close (module);
+		if (path)
+			g_free (path);
+		return NULL;
+	}
+
+	transport = gnome_print_transport_create (get_type, config);
+	if (transport == NULL) {
+		g_warning ("Could not create transport in %s\n", path);
+		g_module_close (module);
+		if (path)
+			g_free (path);
+		return NULL;
+	}
+
+	if (path)
+		g_free (path);
+	
+	return transport;
+}
+
 GnomePrintTransport *
 gnome_print_transport_new (GnomePrintConfig *config)
 {
-	GnomePrintTransport *transport;
-	guchar *drivername;
-	guchar *modulename;
+	GnomePrintTransport *transport = NULL;
+	guchar *module_name = NULL;
+	gint print_to_file = FALSE;
 
 	g_return_val_if_fail (config != NULL, NULL);
 
-	drivername = gnome_print_config_get (config, "Settings.Transport.Backend");
-	if (!drivername) {
-		g_warning ("Settings do not specify transport driver");
-		return NULL;
-	}
+	gnome_print_config_get_boolean (config, "Settings.Output.Job.PrintToFile", &print_to_file);
 
-	transport = NULL;
-
-	modulename = gnome_print_config_get (config, "Settings.Transport.Backend.Module");
-	if (modulename) {
-		static GHashTable *modules = NULL;
-		GModule *module;
-		if (!modules) modules = g_hash_table_new (g_str_hash, g_str_equal);
-		module = g_hash_table_lookup (modules, modulename);
-		if (!module) {
-			gchar *path;
-			path = g_module_build_path (GNOME_PRINT_LIBDIR "/transports", modulename);
-			module = g_module_open (path, G_MODULE_BIND_LAZY);
-			if (module) g_hash_table_insert (modules, g_strdup (modulename), module);
-			g_free (path);
-		}
-		if (module) {
-			gpointer get_type;
-			if (g_module_symbol (module, "gnome_print__transport_get_type", &get_type)) {
-				transport = gnome_print_transport_create (get_type, config);
-			} else {
-				g_warning ("Missing gnome_print__transport_get_type in %s\n", modulename);
-				g_module_close (module);
-			}
-		} else {
-			g_warning ("Cannot open module: %s\n", modulename);
-		}
-		g_free (modulename);
+	if (print_to_file) {
+		module_name = g_strdup ("libgnomeprint-file.so");
 	} else {
-		g_warning ("Unknown transport driver: %s", modulename);
-	}
+		module_name = gnome_print_config_get (config, "Settings.Transport.Backend.Module");
+		if (!module_name) {
+			g_warning ("Could not find \"Settings.Transport.Backend.Module\" using default");
+			module_name = g_strdup ("libgnomeprint-lpr.so");
+		}
+	} 
 
-	g_free (drivername);
+	transport = gnome_print_transport_new_from_module_name (module_name, config);
+
+	g_free (module_name);
 
 	return transport;
 }
@@ -248,6 +327,7 @@ gnome_print_transport_create (gpointer get_type, GnomePrintConfig *config)
 	GnomePrintTransport *transport;
 	GType (* transport_get_type) (void);
 	GType type;
+	GnomePrintReturnCode retval;
 
 	transport_get_type = get_type;
 
@@ -255,8 +335,46 @@ gnome_print_transport_create (gpointer get_type, GnomePrintConfig *config)
 	g_return_val_if_fail (g_type_is_a (type, GNOME_TYPE_PRINT_TRANSPORT), NULL);
 
 	transport = g_object_new (type, NULL);
-	gnome_print_transport_construct (transport, config);
+	retval = gnome_print_transport_construct (transport, config);
+
+	if (retval != GNOME_PRINT_OK) {
+		g_warning ("Error while constructing transport inside transport_create");
+		g_object_unref (G_OBJECT (transport));
+		return NULL;
+	}
 
 	return transport;
 }
 
+
+#define BLOCK_SIZE (1024)
+
+static gint
+gnome_print_transport_real_print_file (GnomePrintTransport *transport, const guchar *file_name)
+{
+	FILE *input;
+	char buffer[1024];
+	int retval;
+
+	input = fopen (file_name, "rb");
+
+	if (input) {
+		int count;
+
+		gnome_print_transport_open (transport);
+
+		while ((count = fread (buffer, 1, BLOCK_SIZE, input))) {
+			retval = gnome_print_transport_write (transport, buffer, count);
+
+			if (retval != count) {
+				fclose(input);
+				return retval;
+			}
+		}
+	}
+
+	fclose(input);
+	retval = gnome_print_transport_close (transport);
+	
+	return retval;
+}
