@@ -27,6 +27,7 @@
 #include <config.h>
 #include <glib.h>
 #include <gmodule.h>
+#include <locale.h>
 
 #include <libgnomeprint/gnome-print-module.h>
 #include <libgnomeprint/gpa/gpa-model.h>
@@ -37,6 +38,15 @@
 #include <cups/cups.h>
 #include <cups/ppd.h>
 
+/* We are protecting the locale due to a known bug in the libcups library. */
+#define LOCALEPROTECTION(c) { char *OLDLOCALE;\
+                              OLDLOCALE = g_strdup(setlocale(LC_ALL, NULL)); \
+                              c; \
+                              setlocale (LC_ALL, OLDLOCALE);\
+                              g_free (OLDLOCALE);\
+                            }
+ 
+
 /* Argument order: id, name */
 
 xmlChar *model_xml_template = 
@@ -80,6 +90,8 @@ xmlChar *model_xml_template =
 "  </Options>"
 "</Model>";
 
+G_MODULE_EXPORT void gpa_module_load_data (GPAPrinter *printer);
+
 static GPANode *
 gpa_model_new_from_xml (char *string)
 {
@@ -258,86 +270,109 @@ get_model (const gchar *printer, ppd_file_t *ppd)
 	return (GPAModel *)model;
 }
 
+
 static gboolean
-append_printer (GPAList *printers_list, const char *name, gboolean is_default)
+append_printer (GPAList *printers_list, const char *name, 
+		gboolean is_default, const gchar *path)
 {
-	GPANode *settings = NULL;
 	GPANode *printer  = NULL;
-	GPAModel *model    = NULL;
-	const char *filename;
-	ppd_file_t *ppd;
 	gboolean retval = FALSE;
 
-	filename = cupsGetPPD (name);
-	ppd = ppdOpenFile (filename);
-	if (ppd == NULL)
-		/* See bug#: 102938 */
-		return FALSE;
+	printer = gpa_printer_new_stub (name, name, path);
 
-	model = get_model (name, ppd);
-	if (model == NULL)
-		goto append_printer_exit;
-
-	settings = gpa_settings_new (model, "Default", "SettIdFromCups");
-	if (settings == NULL)
-		goto append_printer_exit;
-
-	printer = gpa_printer_new (name, name, model, GPA_SETTINGS (settings));
-	if (printer == NULL)
-		goto append_printer_exit;
-
-	if (gpa_node_verify (printer)) {
+	if (printer != NULL && gpa_node_verify (printer)) {
 		gpa_list_prepend (printers_list, printer);
-		if (is_default)
+		if (is_default) {
 			gpa_list_set_default (printers_list, printer);
+			gpa_module_load_data (GPA_PRINTER (printer));
+		}
 		retval = TRUE;
 	}
 
- append_printer_exit:
 	if (retval == FALSE) {
 		g_warning ("The CUPS printer %s could not be created\n", name);
-
 		my_gpa_node_unref (printer);
-		my_gpa_node_unref (GPA_NODE (model));
-		my_gpa_node_unref (settings);
 	}
 
-	if (ppd) {
-		ppdClose (ppd);
-		unlink (filename);
-	}
-	
 	return retval;
 }
 
 static void
-gnome_print_cups_printer_list_append (gpointer printers_list)
+gnome_print_cups_printer_list_append (gpointer printers_list, 
+				      const gchar *path)
 {
-	gboolean is_default;
-	const gchar *def;
-	gchar **printers;
+	/* const gchar *def = cupsGetDefault(); 
+	   not reliable, dests[i].is_default used instead */
+        cups_dest_t *dests = NULL;
 	gint num_printers;
 	gint i;
 
 	g_return_if_fail (printers_list != NULL);
 	g_return_if_fail (GPA_IS_LIST (printers_list));
 	
-	num_printers = cupsGetPrinters (&printers);
-	if (num_printers < 1)
-		return;
-
-	def = cupsGetDefault ();
-	
-	for (i = 0; i < num_printers; i ++) {
-		is_default = (def && !strcmp (printers [i], def));
-		append_printer (GPA_LIST (printers_list),
-				printers [i], is_default);
-	}
+	num_printers = cupsGetDests(&dests);
 	
 	for (i = 0; i < num_printers; i ++)
-		free (printers[i]);
+		append_printer (GPA_LIST (printers_list),
+				dests[i].name, dests[i].is_default,
+				path);
+	
+	cupsFreeDests (num_printers, dests);
+	
+	return;
+}
 
-	free (printers);
+
+/*  ------------- GPA load_data ------------- */
+
+G_MODULE_EXPORT void gpa_module_load_data (GPAPrinter *printer)
+{
+	GPANode *settings = NULL;
+	GPAModel *model    = NULL;
+	char *filename     = NULL;
+	const char *name   = printer->name;
+	ppd_file_t *ppd    = NULL;
+	gboolean success   = FALSE;
+
+	if (printer->is_complete)
+		return;
+
+	
+	LOCALEPROTECTION (filename = g_strdup (cupsGetPPD (name)); 
+			  ppd = ppdOpenFile (filename))
+
+	if (ppd == NULL)
+		goto gpa_module_load_data_exit;
+
+	model = get_model (name, ppd);
+	if (model == NULL)
+		goto gpa_module_load_data_exit;
+
+	settings = gpa_settings_new (model, "Default", "SetIdFromCups");
+	if (settings == NULL)
+		goto gpa_module_load_data_exit;
+
+	success = gpa_printer_complete_stub (printer, model, 
+					     GPA_SETTINGS (settings));
+
+ gpa_module_load_data_exit:
+	if (success == FALSE) {
+		g_warning ("The data for the CUPS printer %s "
+			   "could not be loaded.", name);
+
+		if (model != NULL)
+			my_gpa_node_unref (GPA_NODE (model));
+		if (settings != NULL)
+			my_gpa_node_unref (settings);
+	}
+
+	if (ppd) {
+		ppdClose (ppd);
+		unlink (filename);
+	}
+
+	if (filename)
+		g_free (filename);
 	
 	return;
 }
